from dolfin import *
from petsc4py import PETSc

#PETScOptions.set("mat_superlu_dist_colperm", "NATURAL")
#PETScOptions.set("mat_superlu_dist_colperm", "MMD_AT_PLUS_A")
#PETScOptions.set("mat_superlu_dist_colperm", "MMD_ATA")
#PETScOptions.set("mat_superlu_dist_colperm", "METIS_AT_PLUS_A")
#PETScOptions.set("mat_superlu_dist_colperm", "PARMETIS")
#PETScOptions.set("mat_superlu_dist_rowperm", "NATURAL")
#PETScOptions.set("mat_superlu_dist_rowperm", "NOROWPERM")
#PETScOptions.set("mat_superlu_dist_rowperm", "LargeDiag")
#PETScOptions.set("mat_superlu_dist_iterrefine")
#PETScOptions.set("mat_superlu_dist_statprint ")
#PETScOptions.set("mat_superlu_dist_equil", True)
#PETScOptions.set("mat_superlu_dist_matinput", "DISTRIBUTED")
#PETScOptions.set("mat_superlu_dist_parsymbfact", True)
#PETScOptions.set("mat_superlu_dist_replacetinypivot", False)
#PETScOptions.set("mat_superlu_dist_equil", False)

mesh = UnitSquareMesh(32, 32)
BDM = FiniteElement("BDM", mesh.ufl_cell(), 1)
DG = FiniteElement("DG", mesh.ufl_cell(), 0)
W = FunctionSpace(mesh, BDM * DG)

(sigma, u) = TrialFunctions(W)
(tau, v) = TestFunctions(W)
a = (dot(sigma, tau) + div(tau)*u + div(sigma)*v)*dx
f = Expression("1.0")
L = - f*v*dx

A, b = PETScMatrix(), PETScVector()
assemble(a, tensor=A)
assemble(L, tensor=b)

viewer = PETSc.Viewer().createBinary('mat.dat', 'w')
viewer(A.mat())
viewer = PETSc.Viewer().createBinary('rhs.dat', 'w')
viewer(b.vec())


# mumps solution
w = Function(W)
solve(A, w.vector(), b, 'mumps')
norm_mumps = norm(w)
plot(w.sub(0))
plot(w.sub(1))

# superlu_dist solutio
w = Function(W)
solve(A, w.vector(), b, 'superlu_dist')
norm_superlu_dist = norm(w)
plot(w.sub(0))
plot(w.sub(1))

print 'L2 norm mumps       ', norm_mumps
print 'L2 norm superlu_dist', norm_superlu_dist
interactive()
